VERSION 8000; ; 9114
* -----------------------------------------------------------------------------
* @filedoc
* @file Topo_SymbolPunktBezug.CSX
* @brief Punktreferenzen fr Symbole generieren 
* 
* Dieses Beispielskript generiert Punktreferenzen fr vorhandene Symbole.
* Es werden markierte Symbole der Schicht "Symbole" ausgewertet.  
* Es werden Punkte bercksichtigt, die innerhalb eines Suchkreises um die Koordinate des Symbols liegen.
* Hierbei lsst sich die Punktsuche ber die Kodierungen und Markierung einschrnken.
*  - Wird ein vorh. Punkt gefunden, so wird dieser als Symbolbezugspunkt bernommen.
*  - Wird kein vorh. Punkt gefunden, so wird ein neuer Punkt erzeugt und als Symbolbezugspunkt bernommen (Punktkode = Symbolkode).
* Fr die Punktsuche lassen sich auch nur vorhandene Punkte bercksichtigen. 
*
* -----------------------------------------------------------------------------
* Copyright (C) IB&T
* 
* Gewhrleistung und Nutzung
* 
* Die Nutzung des Skriptes erfolgt stets auf eigene Gefahr. IB&T bernimmt keine
* Gewhr dafr, dass die Nutzung dieses Skriptes zu Ihrem gewnschten Ergebnis
* fhrt. Da CardScripte vorhandene Projektdaten durch Direktzugriff auf die
* jeweiligen Datenbanken auch ohne weitere Sicherheitsabfragen unbemerkt verndern
* oder lschen knnen, erfordert eine Nutzung entsprechende Kenntnisse und groe
* Sorgfalt. IB&T haftet nicht fr eventuell entstehende Datenverluste und
* Folgeschden, die durch die Nutzung von CardScripten entstehen.
*
* -----------------------------------------------------------------------------
* @author JH
* @endfiledoc
* -----------------------------------------------------------------------------
*   8300  18.05.09 JH
*   9144  07.04.21 NR uwandlung von nummerischen in alphanummerische Werte
* @history
* -----------------------------------------------------------------------------


* Vereinbarungen --------------------------------------------------------------

BILDSCHIRM GRAFIK                      | Meldungen im Arbeitsprotokoll ausgeben
$PRFSTUFE 7

* Implizite Deklarationen -----------------------------------------------------

* Schicht und Kodebereich fr die auszuwertenden Symbole
SYMBOLE * !Schicht = "Symbole"
SYMBOLE * !von_SKode = "0"
SYMBOLE * !bis_SKode = "zzzz"

* Filter fr die zu suchenden Punkte
SYMBOLE * P_Mark = 1                             | Schalter fr: 0 = alle Punkte, 1 = nur markierte Punkte
SYMBOLE * !von_PKode = "0"
SYMBOLE * !bis_PKode = "zzzz"
SYMBOLE * SuchR = 0,005                            | Suchradius (Minmax-Bereich) um Symbolkoordinate

SYMBOLE * P_Neu = 1                              | Schalter fr Neupunkte erzeugen: 1 = ja, 0 = nein
SYMBOLE * !PNr_Konst = "PSYM."                   | Konstante fr Punktnummer der neu generierten Punkte
SYMBOLE * LF;!SSKODE;!FINDPKT;!PNAME1;!PNAME2;PI;!NR

* Explizite Deklarationen -----------------------------------------------------

* Systemobjekte
SYMBOLE /TOPOSYS/ TopoSys              | Symbol fr das System der Topografiedaten
SYMBOLE /POINTSYS/ PointSys            | Symbol fr das System der Punkte

* Topografieojekte                    
SYMBOLE /TOPOLAYER/ Schicht            | Symbol fr die Schicht          
SYMBOLE /TOPOSYMBOL/ Symbol            | Symbol fr das Symbol
SYMBOLE /TOPOSYMBOLINDEX/ XSymbol      | Symbol fr Symbolindex  

* Punktobjekte                         
SYMBOLE /POINT/ SPoint; Point          | Symbole fr Punkte 
SYMBOLE /POINTINDEX/ XPoint            | Symbol fr Punktindex

* Hilfsobjekte                
SYMBOLE /COORDINATE/ PKoor             | Symbol fr Hilfspunkt
SYMBOLE /MINMAX/ MiMa                  | Symbol fr Objektausdehnung (Suchbereich)
                                      
* Ausfhrungen -----------------------------------------------------------------

* Ausgeben der Vorgaben/Einstellungen im Protokoll
MELDE "Einstellungen fr die Punktsuche vorhandener Symbole:" 
MELDE " "
MELDE "  Schicht     #                "; !Schicht     
MELDE "  Symbolkode  von #### bis ####"; !von_SKode; !bis_SKode 
MELDE "  Punktkode   von #### bis ####"; !von_PKode; !bis_PKode
MELDE "  Suchradius  #.#              "; SuchR

WENN P_Neu == 1 DANN
  MELDE "  Existiert kein vorhandener Punkt, wird ein neuer Punkt mit der Nummernkonstant '#' generiert (Punktkode = Symbolkode)."; !PNr_Konst
SONST
  MELDE "  Fr die Punktsuche werden nur vorhandene Punkte bercksichtigt." 
WEITER

MELDE " "
MELDE "Liste der vorhandenen Symbole:"
MELDE " "
                                      
* Systemobjekte
TopoSys = Project.GetTopoSys()
PointSys = Project.GetPointSys()

* Hilfsobjekt MinMax-Bereich
MiMa = NEWOBJECT /MINMAX/

* Schichtobjekt 
Schicht = TopoSys.GetLayerByName(!Schicht) 
  * prft, ob Schicht vorhanden ist
  WENN Schicht == $NULL DANN STOP MELDE "Schicht '#' ist nicht vorhanden --> Punktsuche wird abgebrochen!"; !Schicht

* Symbolindexobjekt: Auswahl der Symbole ber Schicht, Kodebereich und Markierung
XSymbol = TopoSys.GetSymbolIndex(!Schicht)                   | liefert Symbolindexobjekt der Schicht
XSymbol.SetAlphaCodeFilter(!von_SKode; !bis_SKode)                  | setzt einen Filter fr den Symbolkode
XSymbol.SetMarkFilter(1)                                     | setzt einen Filter fr die Symbolmarkierung

*** Schleife zum Auslesen der Symbole ber gesetzte Filter 
LF = 0                                                       | Schleifenzhler

* Symbolobjekt
SOLANGE Symbol = XSymbol.GetNext()                           | liest Symbole ber gesetzte Filter aus

  LF = LF + 1
  !SSKode = Symbol.GetAlphaCode()                            | liefert den Symbolkode
  MELDE "  Nr. ####: Punktsuche fr Symbol mit Kode ####"; LF; !SSKode
  
  * Hilfspunktobjekt "PKoor": Koordinate der Symbolposition 
  PKoor = Symbol.GetPosition()                               | liefert die Koordinate der Symbolposition
    
  * Hilfsobjekt MinMax-Bereich: Suchkreis um Symbolkoordinate
  MiMa.Reset()
  MiMa.Add((PKoor.GetX()- SuchR); (PKoor.GetY()- SuchR))
  MiMa.Add((PKoor.GetX()+ SuchR); (PKoor.GetY()+ SuchR))
  * MiMa.Draw(7)                                             | zeichnet den Suchbereich (nur fr Kontrolle einschalten)
  
* Punktobjekt "SPoint": Punkt der Symbolkoordinate
  SPoint = Symbol.GetPoint()                                 | liefert den vorhandenen Punkt der Symbolposition
  
  WENN SPoint == $NULL DANN                                  | wenn Punktbezug nicht vorhanden: sucht Punkt in Punktdatenbank ber Punktindex (XPoint)

    MELDE "      Vorh. Punkt als Symbolbezugspunkt ist nicht vorhanden, Punkt wird gesucht ..."

    * Punktindexobjekt: Auswahl der Punkte innerhalb des Suchradius ber Kodebereich und Markierung
    XPoint = PointSys.GetPointIndex(1)
    XPoint.SetMinMaxFilter(MiMa)                             | setzt einen Filter fr den Suchbereich
    XPoint.SetAlphaCodeFilter(!von_PKode; !bis_PKode)        | setzt einen Filter fr den Punktkode
    XPoint.SetMarkFilter(P_Mark)                             | setzt einen Filter fr die Punktmarkierung

    *** Schleife zum Auslesen der Punkte ber gesetzte Filter
    * Punktobjekt "Point"
    SOLANGE Point = XPoint.GetNext()                         | liest die Punkte ber gesetzte Filter aus
      !FindPkt = Point.GetName()                             | liefert die Punktnummer
      Symbol.SetPoint(Point)                                 | bernimmt den Punkt als Symbolbezugspunkt
      MELDE "      Punkt # als Symbolbezugspunkt bernommen"; !FindPkt 
    ENDE SOLANGE
    *** Ende Schleife
 
    SPoint = Symbol.GetPoint()                               | prft, ob Symbol einen Punkt referenziert

    WENN SPoint == $NULL DANN                                | wenn Punkt nicht gefunden: erzeugt einen neuen Punkt (sofern der Schalter gesetzt ist)

      WENN P_Neu == 1 DANN
      
        MELDE "      Punkt im Suchbereich ist nicht vorhanden, Punkt wird erzeugt ..."
        TEXTGEN !PName1; "######"; LF                        | generiert die Nummer fr neuen Punkt
        !PName1 = !PName1 @ "000000"
        !PName2 = !PNr_Konst + !PName1

        *** Schleife zum Prfen der Punktnummer: Wenn Punktnummer vorhanden, dann nchste Nummer generieren
        SCHLEIFE PI = 1; 999999
          SPoint = PointSys.GetPoint(!PName2)
          WENN SPoint == $NULL DANN ABBRUCH SCHLEIFE
          LF = LF + 1
          TEXTGEN !PName1; "######"; LF 
          !PName1 = !PName1 @ "000000"
          !PName2 = !PNr_Konst + !PName1
        ENDE SCHLEIFE
        *** Ende Schleife

        SPOINT = PointSys.CreatePoint(!PName2; 0)            | erzeugt neuen Punkt mit generierter Nummer 
        * prft, ob freie Punktnummer gefunden wird
        WENN SPoint == $NULL DANN STOP MELDE "      Keine freie Punktnummer gefunden --> Punktgenerierung wird abgebrochen!"
        SPOINT.SetAlphaCode(!SSKode)                         | setzt den Punktkode
        SPoint.SetCoordinate(PKoor)                          | bernimmt die Symbolkoordinate als Punktkoordinate
        Symbol.SetPoint(SPoint)                              | bernimmt den vorhandenen Punkt als Symbolbezugspunkt
        MELDE "      Punkt # neu erzeugt und als Symbolbezugspunkt bernommen"; !PName2
      
      WEITER
    
    WEITER

  SONST                                                      | wenn Punktbezug schon vorhanden: Meldung ausgeben

    !Nr = SPoint.GetName()
    MELDE "      Symbolbezug auf Punkt # schon vorhanden"; !Nr
  
  WEITER
  
  SPoint = $NULL

ENDE SOLANGE